package demo;

import java.awt.event.*;
import java.awt.*;

/**
* Tda slouc pro vytvoen pltna. Zachycuje udlosti "pohyb myi"
* a "stisk tlatka" - proto implementuje uveden rozhran.
* @see java.awt.event.MouseMotionListener
* @see java.awt.event.ActionListener
*/
public class PaintCanvas extends Canvas
implements MouseMotionListener, ActionListener {
   Graphics offGraphics;  // skryt grafick kontext
   Image offImage;        // skryt obrzek (buffer)
   int oldx, oldy;        // "star" souadnice myi

   /** Konstruktor */
   public PaintCanvas() {
      // nastaven barvy pozad na blou
      setBackground(Color.white);

      // registrace sebe (this) jako pjemce udlosti "pohyb myi"
      addMouseMotionListener(this);
   }

   /**
   * Metoda getMinimumSize() mus bt implementovna kvli
   * uren velikosti pro layout manager.
   * @return minimln velikost komponenty (pltna).
   */
   public Dimension getMinimumSize() {
      return new Dimension(300,200);
   }

   /**
   * Metoda getPreferredSize() mus bt implementovna kvli
   * uren velikosti pro layout manager.
   * @return preferovanou velikost komponenty (pltna).
   */
   public Dimension getPreferredSize() {
      return getMinimumSize();
   }

   /**
   * Metoda mouseDragged() je volan pi pohybu a souasnm dren
   * tlatka myi.
   */
   public void mouseDragged(MouseEvent e) {
      // je-li mono provdt vstup, vykresl ru
      if (offGraphics != null)
         offGraphics.drawLine(oldx, oldy, e.getX(), e.getY());

      // uschovn souadnic myi
      oldx = e.getX();
      oldy = e.getY();

      // pekreslit pltno!
      repaint();
   }

   /**
    * Metoda mouseMoved() je volan pi pohybu myi (bez stisku tlatka).
    */
   public void mouseMoved(MouseEvent e) {
      // uschovn souadnic myi
      oldx = e.getX();
      oldy = e.getY();
   }

   /**
    * Metoda actionPerformed() je volan stisku tlatka "Clear".
    */
   public void actionPerformed(ActionEvent e) {
      // zruen skrytho bufferu (bude provedeno vymazn)
      offImage = null;

      // pekreslit pltno!
      repaint();
   }

   /**
    * Metoda update() je volna pi pekreslovn komponenty.
    * Standardn vymae pozad a vol metodu paint().
    * Je pedefinovna, aby nedochzelo k blikn.
    */
   public void update(Graphics g) {
      paint(g);
   }

   /**
    * Metoda paint() provd vykreslen obsahu pltna - pekopruje
    * skryt buffer do grafickho kontextu komponenty (parametr g).
    */
   public void paint(Graphics g) {
      int width = getSize().width;
      int height = getSize().height;

      if (offImage == null) {
         // vytvoen przdnho bufferu
         offImage = createImage(width, height);
         offGraphics = offImage.getGraphics();
      }

      // nastala zmna velikosti komponenty?
      if (offImage.getWidth(this) != width ||
          offImage.getHeight(this) != height) {

         // ano - vytvoit skryt buffer aktuln velikosti...
         Image img = createImage(width, height);
         offGraphics = img.getGraphics();

         // ... a zkoprovat star buffer
         offGraphics.drawImage(offImage,0,0,this);
         offImage = img;
      }

      // zobrazen bufferu do grafickho kontextu komponenty
      g.drawImage(offImage,0,0,this);
   }
}
